#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <net/if.h>
#include <netdb.h>

#include <fcntl.h>
#include <unistd.h>

#include "global.h"
#include "term.h"
#include "disp.h"
#include "tcpcc.h"

#define BUFFSIZE 10

#define AWAIT_NONE	0
#define AWAIT_LEVEL	1
#define AWAIT_KILL	2
#define AWAIT_SCAN	3


void Usage (char *);
long getNetAddr (char *);

int main (int argc, char ** argv)	{

	if (geteuid()!=0)	{
		printf("You have to be root to run this program\n");
		return -1;
		}

	int n, in, nr, cur_index=-1, dst_index=0, cur_panel=0,m;
	int awaiting=AWAIT_NONE,
		 level=0,
		 verbose=0,
		 wait=0;
	char timeout_char [6]; 
	int timeout_index=0, timeout_kill=10,
	    timeout_scan=10;
	
	int interactive=0;
	
	char buf [BUFFSIZE];
	struct pollfd pl[1];
	char c;
	char interface [16]="",
			broadcast [16]="", 
			netmask [16]="";

	char srcip [LEFT_PANEL][255], 
		  dstip [RIGHT_PANEL][255];
	
//	char temp_string [16];
	unsigned long source_ip=0,
		destination_ip=0;
	unsigned int source_port=0,
		destination_port=0;
	
	int src_ip_count=0, 
	    dst_ip_count=0;

	char message [100];
	unsigned int pkts;

	struct in_addr adr;

	while ((c=getopt(argc,argv,"i:m:b:l:k:s:S:D:p:P:Ihwv"))!=EOF)  {
   	switch (c)  {
   		case 'i': strncpy(interface,optarg,10);
    		   break;
			case 'b': strncpy(broadcast,optarg,15);
				break;
			case 'm': strncpy(netmask,optarg,15);
				break;
			case 'I': interactive=1;
				break;
			case 'l': level=atoi(optarg);
				break;
			case 'k': timeout_kill=atoi(optarg);
				break;
			case 's': timeout_scan=atoi(optarg);
				break;
			case 'S': source_ip=getNetAddr(optarg);
				break;
			case 'D': destination_ip=getNetAddr(optarg);
				break;
			case 'p': source_port=htons(atoi(optarg));
				break;
			case 'P': destination_port=htons(atoi(optarg));
				break;
			case 'v': verbose=1;
				break;
			case 'w': wait=1;
				break;
			case 'h':
    		default: Usage (*argv);
     		   return -1;
  			   }
 		  }

	Tcpcc tcpcc;
			
	tcpcc.initialize(interface,broadcast,netmask);

	if (interactive==0)	{
		if ((!destination_ip)&&(!destination_port)&&(!source_port)&&(source_ip)&&(level!=4))	{
			adr.s_addr=source_ip;
			printf ("Assuming level 4 (killing all connections from %s).\n", inet_ntoa(adr));
			fflush(stdout);
			level=4;
			}
		else if ((!destination_ip)&&(source_ip)&&(!source_port)&&(destination_port)&&(level!=3))	{
			adr.s_addr=source_ip;
			printf ("Assuming level 3 (killing all connections from %s to any host and port %d).\n", 
				inet_ntoa(adr), htons(destination_port));
			fflush(stdout);
			level=3;
			}
		else if ((!destination_port)&&(source_ip)&&(!source_port)&&(destination_ip)&&(level!=2))	{
			adr.s_addr=source_ip;
			printf ("Assuming level 2 (killing all connections from %s to ", inet_ntoa(adr));
			adr.s_addr=destination_ip;
			printf ("%s)\n", inet_ntoa(adr));
			fflush(stdout);
			level=2;
			}
		else if ((destination_port)&&(destination_ip)&&(source_ip)&&(!source_port)&&(level!=1))	{
			adr.s_addr=source_ip;
			printf ("Assuming level 1 (killing all connections from %s to ", inet_ntoa(adr));
			adr.s_addr=destination_ip;
			printf ("%s:%d)\n",inet_ntoa(adr), ntohs(destination_port));
			fflush(stdout);
			level=1;
			}
		else if ((destination_port)&&(destination_ip)&&(source_ip)&&(source_port)&&(level!=0))	{
         adr.s_addr=source_ip;
			printf("Assuming level 0 (killing all connections from %s:%d ", inet_ntoa(adr), ntohs(source_port));
			adr.s_addr=destination_ip;
			printf("to %s:%d)\n",inet_ntoa(adr), ntohs(destination_port));
			fflush(stdout);
			level=0;
			}
		else	{
			Usage(*argv);
			return -1;
			}
		if (wait==0)	{// dont wait for child pid to end.
			if (verbose) {
				printf("forking\n");
				fflush(stdout);
				}
			tcpcc.prepare2kill(source_ip, source_port, destination_ip, destination_port, timeout_kill, level);
			}
		else {	
			if (verbose)	{
				printf("waiting for packet and senting RST\n");
				fflush(stdout);
				}
			tcpcc.killhim(source_ip,source_port,destination_ip,destination_port, timeout_kill, level);
			if (verbose)	{
				printf("sending done\n");
				fflush(stdout);
				}
			}	
		return 0;
		}
	
	if ((in=open("/dev/stdin",O_RDONLY))==-1)	{
		perror("stdin open");
		return -1;
		}
	
	pl[0].fd=in;
	pl[0].events=POLLIN || POLLPRI;

//	tcpcc.find_active();
	
	Display display;
	display.display_header();
	
	for (;;) {
		nr = poll(pl,1,-1);
		if (pl[0].revents && POLLIN)  {  // przyszedl message ze stdin
		   n=read(pl[0].fd,buf,BUFFSIZE);
			switch(*buf)	{
				case 0x1b :
					if ((awaiting==AWAIT_KILL)||(awaiting==AWAIT_SCAN))	timeout_index=0;
					awaiting=AWAIT_NONE;
					display.status_bar("");
					if (tcpcc.ip_count>0)	{
						switch (*(buf+2))	{
							case KEY_LEFT:
								if (cur_panel>0) cur_panel--;
				               display.refresh_screen(srcip, dstip, src_ip_count, dst_ip_count, cur_index, dst_index, cur_panel);
								break;
							case KEY_RIGHT:
								if (cur_panel<1) {
									cur_panel++;
									dst_index=0;
									}
			               display.refresh_screen(srcip, dstip, src_ip_count, dst_ip_count, cur_index, dst_index, cur_panel);
								break;
							case KEY_DOWN:
								if (cur_panel==0) {
									if (cur_index<src_ip_count-1) cur_index++;
									}
								else {
									if (dst_index<dst_ip_count-1) dst_index++;
									}
								display.clear_panels(src_ip_count, dst_ip_count);	
								tcpcc.translate(srcip,dstip,&src_ip_count, &dst_ip_count, cur_index);
			               display.refresh_screen(srcip, dstip, src_ip_count, dst_ip_count, cur_index, dst_index, cur_panel);
								break;
							case KEY_UP:
								if (cur_panel==0)	{
									if (cur_index>0) cur_index--;
									}
								else {
									if (dst_index>0) dst_index--;
									}
								display.clear_panels(src_ip_count, dst_ip_count);	
								tcpcc.translate(srcip,dstip,&src_ip_count, &dst_ip_count, cur_index);
			               display.refresh_screen(srcip, dstip, src_ip_count, dst_ip_count, cur_index, dst_index, cur_panel);
								break;
							} // specjal combinations
						}	// if ip_count
					break;
				case 'q':
				case 'Q':
					display.close_graph();
					return -1;
					break;
				case 'k':
				case 'K':
					if ((awaiting==AWAIT_KILL)||(awaiting==AWAIT_SCAN))	timeout_index=0;
					awaiting=AWAIT_NONE;
					if (cur_panel==1) tcpcc.prepare2killint(cur_index, dst_index, timeout_kill,level);
					tcpcc.translate(srcip,dstip,&src_ip_count, &dst_ip_count, cur_index);
               display.refresh_screen(srcip, dstip, src_ip_count, dst_ip_count, cur_index, dst_index, cur_panel);
					break;
				case 'a':
				case 'A': // show active connections
					if ((awaiting==AWAIT_KILL)||(awaiting==AWAIT_SCAN))	timeout_index=0;
					awaiting=AWAIT_NONE;
					display.status_bar("Finding active connections...");
					pkts = tcpcc.find_active(timeout_scan);
					m = tcpcc.generate_ips_struct();
					snprintf(message, STATUS_BAR_SIZE, "Received %d packets matching %d connections.",pkts,m);
					display.status_bar(message);
					cur_index=0;
					tcpcc.translate(srcip,dstip,&src_ip_count, &dst_ip_count, cur_index);
					display.refresh_screen(srcip, dstip, src_ip_count, dst_ip_count, cur_index, dst_index, cur_panel);
					break;
				case 'l':
				case 'L':
					if ((awaiting==AWAIT_KILL)||(awaiting==AWAIT_SCAN))	timeout_index=0;
					awaiting=AWAIT_LEVEL;
					snprintf(message,STATUS_BAR_SIZE,"Enter kill level <0-4> [%d].",level);
					display.status_bar(message);
					break;
				case 's':
				case 'S':
					timeout_index=0;
					snprintf(message,STATUS_BAR_SIZE,"Enter scan timeout [%d]:",timeout_scan);
					display.status_bar(message);
					awaiting=AWAIT_SCAN;
					break;
				case 't':
				case 'T':
					timeout_index=0;
					snprintf(message,STATUS_BAR_SIZE,"Enter kill timeout [%d]:",timeout_kill);
					display.status_bar(message);
					awaiting=AWAIT_KILL;
					break;
				case '0':
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
				case '9':
					if (awaiting==AWAIT_LEVEL)	{ // avaiting for level
						if ((((*buf)>='1')&&((*buf)<='3'))||(*buf)=='0')	{
							level=(*buf)-48;
							awaiting=AWAIT_NONE;
							snprintf(message,STATUS_BAR_SIZE,"Level %d set.",level);
							display.status_bar(message);
							}
						}
					else if ((awaiting==AWAIT_KILL)||(awaiting==AWAIT_SCAN)) {	// avaiting for timeout
						if (timeout_index<5) {
							*(timeout_char+timeout_index++)=*buf;
							*(timeout_char+timeout_index)='\0';
							snprintf(message,STATUS_BAR_SIZE," %s",timeout_char);
							display.status_bar_append(message);
							}
						}
					break;
				case KEY_ENTER:
					if (awaiting==AWAIT_LEVEL)	{
						snprintf(message,STATUS_BAR_SIZE,"Level %d set.",level);
						}
					else if (timeout_index!=0)	{
						*(timeout_char+timeout_index)='\0';
						if (awaiting==AWAIT_KILL)	timeout_kill=atoi(timeout_char);
						else if (awaiting==AWAIT_SCAN)	
							timeout_scan = (timeout_scan=atoi(timeout_char))==0 ? 1 : timeout_scan;
						}
					if (awaiting==AWAIT_KILL)	snprintf(message,STATUS_BAR_SIZE,"Kill timeout %d set.",timeout_kill);
					else if (awaiting==AWAIT_SCAN)	snprintf(message,STATUS_BAR_SIZE,"Scan timeout %d set.",timeout_scan);

					display.status_bar(message);
					awaiting=AWAIT_NONE;
					break;
				}
			}
		} // main for loop

return 0;
}


long getNetAddr (char * nazwa)	{
struct hostent * host;
long netaddr;
struct sockaddr_in dest;
if ((netaddr=inet_addr(nazwa))!=-1) return netaddr;
	
if ((host = gethostbyname(nazwa))==NULL) return -1; 
else { 
	bcopy (host->h_addr, (caddr_t) &dest.sin_addr, host->h_length);
	return dest.sin_addr.s_addr;
	}
return -1;
}


void Usage (char * n)	{
	printf("\nLocal Tcp Control Center version 1.0.2 by Pawel Prokop (pablo@wizard.ae.krakow.pl)\n\n");
	printf("usage : ltcc [-Ihwv] [-i interface] [-b broadcast] [-m netmask] [-l level]\n");
	printf("             [-k kill timeout] [-s scan timeout] [-S address] [-p port]\n");
	printf("             [-D address] [-P port]\n\n");
	printf("  -I : interactive mode\n");
	printf("  -h : this help screen\n");
	printf("  -w : wait for end of killing (do not fork)\n");
	printf("  -v : verbose mode\n\n");
	printf("  -i interface : ethernet interface ex. eth0, eth1...\n");
	printf("  -b broadcast : broadcast address for network listen to. (default: from interface)\n");
	printf("  -m netmask   : netmask for network. (default take from interface)\n");
	printf("  -l level     : kill level describes what have to match in sniffed packet\n");
	printf("                 for RST flag to be set and spooffed packet to be sent\n");
	printf("     level=0 : source ip & source port & destination ip & destination port\n");
	printf("     level=1 : source ip & destination ip & destination port\n");
	printf("     level=2 : source ip & destination ip\n");
	printf("     level=3 : source ip & destination port\n");
	printf("     level=4 : source ip only - the most destructive ;-)\n");
	printf("  -k timeout : (sec) time of child process living\n");
	printf("  -s timeout : (sec) time of sniffing packets in LAN (interactive mode)\n"); 
	printf("  -S address : source IP address\n");
	printf("  -p port    : source port\n");
	printf("  -D address : destination IP address\n");
	printf("  -P port    : destination port\n\n");

exit (0);
}
