#include <termios.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>

#define _TERM_H

#define ESC       "\033"
#define COLOR(N)  ESC "[3" #N "m"

#define SAVE      ESC "7"  /* save current state      */
#define BOLD      ESC "[1m"
#define RESET     ESC "[0m"   /* reset attr to their defaults  */

/* restore most recently saved state */
#define RESTORE   ESC "8"
#define BACKSPACE "\b \b"

/* key codes */
#define CTRL_A	0x01
#define CTRL_E	0x05
#define CTRL_J	0x0a
#define KEY_ENTER	0x0d
#define CTRL_Q	0x11
#define CTRL_W 0x17
#define CTRL_R	0x12
#define CTRL_T	0x14
#define CTRL_Y 0x19
#define CTRL_U	0x15
#define CTRL_I 0x09
#define CTRL_O 0x0f
#define CTRL_P	0x10
#define CTRL_S	0x13
#define CTRL_D	0x04
#define CTRL_F	0x06
#define CTRL_G	0x07
#define CTRL_H	0x08
#define CTRL_K	0x0b
#define CTRL_L	0x0c
#define CTRL_X	0x18
#define CTRL_V 0x16
#define CTRL_B	0x02
#define CTRL_N 0x0e
#define CTRL_1d	0x1d

/* extended key code definitions */
#define KEY_HOME	0x31
#define KEY_DEL	0x33
#define KEY_END	0x34
#define KEY_UP		0x41
#define KEY_DOWN	0x42
#define KEY_RIGHT	0x43
#define KEY_LEFT	0x44
#define KEY_BACK	0x7f
#define KEY_PRINT	0x1c



class Terminal	{
	public:
		Terminal ();
		void Reset ();
		void term_raw();
		void term_rest();
		void goto_xy(int,int);
		void set_cols();
		void clear();
		unsigned int cols, rows;
		struct termios tio;
	};
